// Alarm_Clock Joe Pardue December 20, 2012
// This is the Arduino side software for the January Nuts&Volts article:
// Smiley's Workshop 54 Proto Shield Alarm Clock.
// You may find a version of this article on my blog at www.smileymicros.com\blog
//
// This is an educational demonstration and is neither the best way to
// do things nor does it have more than cursory error catching.
//
// Most of the DS1307 timekeeping items were taken from RTCLib provided by
// Jean-Claude Wippler of www.jeelabs.com
//
// Date and time functions using a DS1307 RTC connected via I2C and Wire lib
//
// I organized this code in four modules:
// 1. Alarm_Clock - this module - headers, variables, defines, setup(), loop(), and timer interrupt.
// 2. Commander - parses commands from the PC.
// 3. Date_Time - functions for using dates and times
// 4. Alarm_Clock - alarm service functions.

#include <Wire.h> // Arduino I2C library 
#include "RTClib.h" // DS1307 Real Time Clock library
#include <TimerOne.h> // Timer 1 library
#include <EEPROM.h>

#define DEBUG // Comment out for production cod

//
// Button debounce stuff from Arduino IDE example
//
// constants won't change. They're used here to 
// set pin numbers:
const int buttonPin = 9;     // the number of the pushbutton pin
const int ledPin =  7;      // the number of the LED pin
// Variables will change:
int ledState = HIGH;         // the current state of the output pin
int buttonState;             // the current reading from the input pin
int lastButtonState = LOW;   // the previous reading from the input pin
// The following variables are long's because the time, measured in miliseconds,
// will quickly become a bigger number than can be stored in an int.
long lastDebounceTime = 0;  // the last time the output pin was toggled
long debounceDelay = 50;    // the debounce time; increase if the output flickers

// DateTime structure initializations
DateTime present (0, 0, 0, 0, 0, 0);

// defines for the EEPROM addresses
#define A1A 0
#define A2A 6
#define A3A 13
#define A4A 20
#define A5A 27

struct alarm {
  DateTime datetime;
  uint8_t type;
  uint8_t is_set;
  uint8_t is_tripped;
};

alarm alarm1;
alarm alarm2;
alarm alarm3;
alarm alarm4;
alarm alarm5;


RTC_DS1307 RTC; // Used by RTClib

// Communication Constants
#define TERMINATOR '!' // Terminator character
#define SEPARATOR ',' // Separator character
#define MAX_COMMAND_LENGTH 16
#define MAX_COMMANDS 8
#define MAX_MESSAGE MAX_COMMAND_LENGTH * MAX_COMMANDS
#define MAX_NUMBER_SIZE 6

// Communication arrays
byte commandArray[MAX_COMMAND_LENGTH];
byte messageArray[MAX_MESSAGE];
// Communication counts
int commandArrayCount = 0;
int messageArrayCount = 0;

// A timer is used to set the check_alarm flag  
int check_alarm = 0;
int check_alarm_count = 0;


int three_short_beep = 0;
int three_short[] = { 1, 0, 1, 0, 1, 0, 2};
int three_long_beep = 0;
int three_long[] = { 1, 1, 0, 1, 1, 0, 1, 1, 0, 2};
int sos_beep = 0;
int sos[] = { 1, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 0, 2};
int warble3_beep = 0;
int warble3[] = { 1, 2, 0, 1, 2, 0, 1, 2, 0, 3};
int beep_count = 0;

// Calibration variables
uint32_t calibration;
uint32_t cal_start_time;
uint8_t cal_is_set;
uint8_t addsubtract;
#define CAL_EEPROM_ADDRESS 34 // follows alarms

// The loop can call functions that have already been called
// this chokes the system, I've added a choke variable to 
// let the loop calls know if a function is already running.
//int choke = 0;
  
void setup()
{
  // Set up the serial port
  Serial.begin(57600);
  
  // Say howdy
  Serial.println("Alarm_Clock rev. 0.09"); 
 
  initializeMessageArray(); // Initialize message array
  initializeDS1307RTC(); // Initialize the external RTC
  initializeAlarms(); // Initailize the alarms

  // initialize timer1
  Timer1.initialize(250000); // trips 4 times a second
  Timer1.attachInterrupt(myTimer1); 

#if defined(DEBUG)  
  if(!RTC.isrunning()) Serial.println("RTC is NOT running.");  
  else Serial.println("RTC is running."); 
#endif   
}

void loop()
{
  // Check for messages from the PC
  if(Serial.available() )
  {
    loadMessageArray();
  }

  // timer sets check_alarm once per second
  // check the alarm if it is set
  if(check_alarm){      
    if(cal_is_set){
      go_calibration();
    }    
    else if(alarm1.is_set){
     go_alarm1();
    }
    else if(alarm2.is_set){
      go_alarm2();
    } 
    else if(alarm3.is_set){
      go_alarm3();
    }  
    else if(alarm4.is_set){
      go_alarm4();
    }
    else if(alarm5.is_set){
      go_alarm5();
    }
    check_alarm = 0;
  }
 
// Button debounce stuff from Arduino IDE example
   //Check for button press
  // read the state of the switch into a local variable:
  int reading = digitalRead(buttonPin);

  // check to see if you just pressed the button 
  // (i.e. the input went from LOW to HIGH),  and you've waited 
  // long enough since the last press to ignore any noise:  

  // If the switch changed, due to noise or pressing:
  if (reading != lastButtonState) {
    // reset the debouncing timer
    lastDebounceTime = millis();
  } 
  
  if ((millis() - lastDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer
    // than the debounce delay, so take it as the actual current state:
    buttonState = reading;
  }
  
  // set the LED using the state of the button:
  digitalWrite(ledPin, buttonState);
  
  // Clear all tripped alarms if button pressed.
  if(buttonState){
   alarm1.is_tripped = 0;
   alarm2.is_tripped = 0;
   alarm3.is_tripped = 0;
   alarm4.is_tripped = 0;
   alarm5.is_tripped = 0;     
  }

  // save the reading.  Next time through the loop,
  // it'll be the lastButtonState:
  lastButtonState = reading; 
}



// Rather than use the loop to check the alarm continuously
// and thereby adding quite a load on the processor, we will
// use a timer set to check the alarms once per second
void myTimer1()
{ 
  if(check_alarm_count++ >= 3){
    check_alarm = 1;
    check_alarm_count = 0;
  }

  if(three_short_beep){    
     if(three_short[beep_count] == 0) noTone(8);
     else if(three_short[beep_count] == 1) tone(8,4300);
     else if(three_short[beep_count] == 2){
       three_short_beep = 0;
       beep_count = -1;
     }
     beep_count++;
  }
  if(three_long_beep){
     if(three_long[beep_count] == 0) noTone(8);
     else if(three_long[beep_count] == 1) tone(8,4300);
     else if(three_long[beep_count] == 2){
       three_long_beep = 0;
       beep_count = -1;
     }
     beep_count++;
  }  
  if(sos_beep){
     if(sos[beep_count] == 0) noTone(8);
     else if(sos[beep_count] == 1) tone(8,4300);
     else if(sos[beep_count] == 2){
       sos_beep = 0;
       beep_count = -1;
     }
     beep_count++;
  }       
  if(warble3_beep){
     if(warble3[beep_count] == 0) noTone(8);
     else if(warble3[beep_count] == 1) tone(8,4000);
     else if(warble3[beep_count] == 2) tone(8,4300);
     else if(beep_count >= 3){
       warble3_beep = 0;
       beep_count = -1;
     }
     beep_count++;
  } 
}




